"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BullJob = void 0;
const bull_1 = __importDefault(require("bull"));
const instance_1 = require("../instance");
const utils_1 = require("../utils/utils");
var JobNames;
(function (JobNames) {
    JobNames["CronJob"] = "CronJob";
    JobNames["CronLikeJob"] = "CronLikeJob";
    JobNames["DelayedJob"] = "DelayedJob";
})(JobNames || (JobNames = {}));
class BullJob {
    #queue;
    constructor() {
        this.#queue = new bull_1.default(instance_1.Instance.get().settings.bullQueueName, instance_1.Instance.get().settings.redisURI);
    }
    static #getNewId() {
        return [Date.now(), utils_1.Random.string()].join(':');
    }
    async addDelayedJob(data, delayInMs) {
        const job = await this.#queue.add(JobNames.DelayedJob, data, {
            jobId: BullJob.#getNewId(),
            delay: delayInMs,
            removeOnComplete: true,
            backoff: 1000,
            attempts: 3
        });
        return job.id.toString();
    }
    async addCronLikeJob(data, cron, tz) {
        const job = await this.#queue.add(JobNames.CronLikeJob, data, {
            jobId: BullJob.#getNewId(),
            repeat: { cron, ...(tz ? { tz } : {}) },
            removeOnComplete: true,
            backoff: 1000,
            attempts: 3
        });
        return job.opts?.repeat?.key ?? '';
    }
    async removeDelayedJob(jobId) {
        const job = await this.#queue.getJob(jobId);
        if (job)
            await job.discard();
    }
    async removeCronLikeJob(jobKey) {
        await this.#queue.removeRepeatableByKey(jobKey);
    }
    async retryAllFailedJobs() {
        const failedJobs = await this.#queue.getFailed();
        await Promise.all(failedJobs.map((job) => job.retry()));
    }
    async startProcessingQueues(crons, callbacks) {
        await this.#cleanup();
        await Promise.all(crons.map(({ cron, name }) => this.#addCronJob(name, cron)));
        await Promise.all([
            this.#queue.process(JobNames.DelayedJob, async (job) => await callbacks.onDelayed?.(job.data)),
            this.#queue.process(JobNames.CronJob, async (job) => await callbacks.onCron?.(job.data.type)),
            this.#queue.process(JobNames.CronLikeJob, async (job) => await callbacks.onCronLike?.(job.data))
        ]);
    }
    async #addCronJob(type, cron) {
        const job = await this.#queue.add(JobNames.CronJob, { type }, {
            repeat: { cron },
            removeOnComplete: true,
            backoff: 1000,
            attempts: 3
        });
        return job.id.toString();
    }
    async #cleanup() {
        await this.retryAllFailedJobs();
        const repeatableJobs = await this.#queue.getRepeatableJobs();
        await Promise.all(repeatableJobs
            .filter((job) => job.name === JobNames.CronJob)
            .map((job) => this.#queue.removeRepeatableByKey(job.key)));
    }
}
exports.BullJob = BullJob;
