"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoDbChange = void 0;
const mongoose_1 = __importDefault(require("mongoose"));
const exit_1 = require("../../exit");
const instance_1 = require("../../instance");
const retry_1 = require("../../utils/retry");
const validations_1 = require("../../validations");
const _instance_1 = require("../_instance");
const debezium_1 = require("../debezium");
class MongoDbChange extends _instance_1.DbChange {
    #started = false;
    #model;
    constructor(model, callbacks, mapper) {
        super(callbacks, mapper);
        this.#model = model;
    }
    async start() {
        if (this.#started)
            return;
        this.#started = true;
        const model = this.#model;
        const dbName = model.db.name;
        const colName = model.collection.name;
        const dbColName = `${dbName}.${colName}`;
        const topic = `${debezium_1.TopicPrefix}.${dbColName}`;
        await (0, retry_1.retry)(async () => {
            const { hosts, replicaSet, credentials } = model.collection.conn.getClient().options;
            const started = await this._setup(topic, {
                'connector.class': 'io.debezium.connector.mongodb.MongoDbConnector',
                'capture.mode': 'change_streams_update_full_with_pre_image',
                'mongodb.connection.string': `mongodb://${hosts.map((h) => `${h.host}:${h.port}`).join(',')}/?replicaSet=${replicaSet}`,
                ...(credentials ? {
                    'mongodb.user': credentials.username,
                    'mongodb.pass': credentials.password,
                    'mongodb.auth-source': credentials.source
                } : {}),
                'collection.include.list': dbColName
            });
            const hexId = '5f5f65717569707065645f5f';
            const TestId = new mongoose_1.default.Types.ObjectId(hexId);
            const hydrate = (data) => data._id ? model.hydrate({
                ...data, _id: makeId(data._id['$oid'] ?? data._id)
            }).toObject({ getters: true, virtuals: true }) : undefined;
            instance_1.Instance.get().eventBus.createSubscriber(topic, async (data) => {
                const op = data.op;
                let before = JSON.parse(data.before ?? 'null');
                let after = JSON.parse(data.after ?? 'null');
                if (before)
                    before = hydrate(before);
                if (after)
                    after = hydrate(after);
                if (before?.__id === TestId || after?.__id === TestId)
                    return;
                if (op === 'c' && this.callbacks.created && after)
                    await this.callbacks.created({
                        before: null,
                        after: this.mapper(after)
                    });
                else if (op === 'u' && this.callbacks.updated && before && after)
                    await this.callbacks.updated({
                        before: this.mapper(before),
                        after: this.mapper(after),
                        changes: validations_1.Validation.Differ.from(validations_1.Validation.Differ.diff(before, after))
                    });
                else if (op === 'd' && this.callbacks.deleted && before)
                    await this.callbacks.deleted({
                        before: this.mapper(before),
                        after: null
                    });
            });
            if (!started) {
                await model.findByIdAndUpdate(TestId, { $set: { colName } }, { upsert: true });
                await model.findByIdAndDelete(TestId);
                await instance_1.Instance.get().logger.warn(`Waiting for db changes for ${dbColName} to start...`);
                throw new Error(`Wait a few minutes for db changes for ${dbColName} to initialize...`);
            }
        }, 5, 60_000)
            .catch((err) => (0, exit_1.exit)(err.message));
    }
}
exports.MongoDbChange = MongoDbChange;
const makeId = (id) => {
    try {
        return new mongoose_1.default.Types.ObjectId(id);
    }
    catch {
        return id;
    }
};
