"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseMongodbQueryParams = void 0;
const instance_1 = require("../../instance");
const query_1 = require("../query");
const parseMongodbQueryParams = async (model, params) => {
    const query = [];
    const whereType = Object.values(query_1.QueryKeys).indexOf(params.whereType) !== -1 ? params.whereType : query_1.QueryKeys.and;
    const authType = Object.values(query_1.QueryKeys).indexOf(params.authType) !== -1 ? params.authType : query_1.QueryKeys.and;
    const where = buildWhereQuery(params.where ?? [], whereType);
    if (where)
        query.push(where);
    const auth = buildWhereQuery(params.auth ?? [], authType);
    if (auth)
        query.push(auth);
    if (params.search && params.search.fields.length > 0) {
        const search = params.search.fields.map((field) => ({
            [field]: {
                $regex: new RegExp(params.search.value, 'i')
            }
        }));
        query.push({ $or: search });
    }
    const totalClause = {};
    if (query.length > 0)
        totalClause['$and'] = query;
    const sort = params.sort?.map((p) => [p.field, p.desc ? 'desc' : 'asc']) ?? [];
    const all = params.all ?? false;
    const settings = instance_1.Instance.get().settings;
    const limit = Number(params.limit) <= settings.paginationDefaultLimit ? Number(params.limit) : settings.paginationDefaultLimit;
    let page = Number.isNaN(Number(params.page)) ? 0 : Number(params.page);
    page = page < 1 ? 1 : page;
    const total = await model.countDocuments(totalClause).catch(() => {
        throw new Error('Error querying database');
    });
    let builtQuery = model.find(totalClause).lean({ virtuals: true, getters: true, defaults: true });
    if (sort.length)
        builtQuery = builtQuery.sort(Object.fromEntries(sort));
    if (!all && limit) {
        builtQuery = builtQuery.limit(limit);
        if (page)
            builtQuery = builtQuery.skip((page - 1) * limit);
    }
    const results = await builtQuery.catch(() => {
        throw new Error('Error querying database');
    });
    const start = 1;
    const last = Math.ceil(total / limit) || 1;
    const next = page >= last ? null : page + 1;
    const previous = page <= start ? null : page - 1;
    return {
        pages: { start, last, next, previous, current: page },
        docs: { limit, total, count: results.length },
        results: results
    };
};
exports.parseMongodbQueryParams = parseMongodbQueryParams;
const buildWhereQuery = (params, key = query_1.QueryKeys.and) => {
    const where = (Array.isArray(params) ? params : []).map((param) => {
        if (Object.values(query_1.QueryKeys).includes(param.condition))
            return buildWhereQuery(param.value, param.condition);
        const { field } = param;
        const checkedField = field === 'id' ? '_id' : (field ?? '');
        const checkedValue = param.value === undefined ? '' : param.value;
        const checkedCondition = Object.keys(query_1.Conditions).indexOf(param.condition) > -1 ? param.condition : query_1.Conditions.eq;
        return ({
            field: checkedField,
            value: checkedValue,
            condition: checkedCondition,
            isWhere: true
        });
    }).filter((c) => c).map((c) => {
        if (c.isWhere)
            return {
                [`${c.field}`]: { [`$${c.condition}`]: c.value }
            };
        else
            return c;
    });
    return where.length > 0 ? { [`$${key}`]: where } : null;
};
