"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Instance = void 0;
const pino_1 = __importDefault(require("pino"));
const bull_1 = require("../bull");
const redis_cache_1 = require("../cache/types/redis-cache");
const mongoose_1 = require("../db/mongoose");
const kafka_1 = require("../events/kafka");
const exit_1 = require("../exit");
const server_1 = require("../server");
const settings_1 = require("./settings");
class Instance {
    static #initialized = false;
    static #instance;
    #settings = { ...settings_1.defaulInstanceSetting };
    #logger = null;
    #job = null;
    #cache = null;
    #eventBus = null;
    #server = null;
    #dbs = null;
    constructor() {
    }
    get logger() {
        return this.#logger ||= Instance.createLogger();
    }
    get job() {
        return this.#job ||= new bull_1.BullJob();
    }
    get cache() {
        return this.#cache ||= new redis_cache_1.RedisCache();
    }
    get eventBus() {
        return this.#eventBus ||= new kafka_1.KafkaEventBus();
    }
    get server() {
        return this.#server ||= server_1.serverTypes[this.settings.server]();
    }
    get dbs() {
        return this.#dbs ||= { mongo: new mongoose_1.MongoDb() };
    }
    get listener() {
        return this.server.listener;
    }
    get settings() {
        return this.#settings;
    }
    static createLogger() {
        const defaultLogLevel = 'info';
        return (0, pino_1.default)({
            level: Instance.#initialized ? Instance.get().settings?.logLevel ?? defaultLogLevel : defaultLogLevel,
            serializers: {
                err: pino_1.default.stdSerializers.err,
                req: pino_1.default.stdSerializers.req,
                res: pino_1.default.stdSerializers.res,
            },
        });
    }
    static initialize(settings) {
        Instance.#initialized = true;
        const instanceSettings = Instance.get().settings;
        Object.entries(settings).forEach(([key, value]) => {
            instanceSettings[key] = value;
        });
    }
    static get() {
        if (!this.#initialized)
            return (0, exit_1.exit)('Has not been initialized. Make sure initialize is called before you get an instance');
        if (!Instance.#instance)
            Instance.#instance = new Instance();
        return Instance.#instance;
    }
    async startConnections() {
        try {
            await Instance.get().cache.start();
            await Instance.get().listener.start();
            await Promise.all(Object.values(Instance.get().dbs).map(async (db) => {
                await db.start();
                await db.startAllDbChanges();
                (0, exit_1.addWaitBeforeExit)(db.close);
            }));
            await Instance.get().eventBus.startSubscribers();
            (0, exit_1.addWaitBeforeExit)(Instance.get().cache.close);
        }
        catch (error) {
            (0, exit_1.exit)(`Error starting connections: ${error}`);
        }
    }
}
exports.Instance = Instance;
