"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Listener = void 0;
const path_to_regexp_1 = require("path-to-regexp");
const instance_1 = require("../instance");
const server_1 = require("../server");
const tokens_1 = require("../utils/tokens");
var EmitTypes;
(function (EmitTypes) {
    EmitTypes["created"] = "created";
    EmitTypes["updated"] = "updated";
    EmitTypes["deleted"] = "deleted";
})(EmitTypes || (EmitTypes = {}));
const EmitterEvent = '__listener_emitter';
class Listener {
    #socket;
    #callers;
    #routes = {};
    #subscriber = instance_1.Instance.get().eventBus.createSubscriber(EmitterEvent, async (data) => {
        this.#socket.to(data.channel).emit(data.channel, data);
    }, { fanout: true });
    #publisher = instance_1.Instance.get().eventBus.createPublisher(EmitterEvent);
    constructor(socket, callers) {
        this.#socket = socket;
        this.#callers = callers;
        this.#setupSocketConnection();
    }
    async start() {
        await this.#subscriber.subscribe();
    }
    async created(channels, data) {
        await this.#emit(channels, EmitTypes.created, { after: data.toJSON(), before: null });
    }
    async updated(channels, { after, before }) {
        await this.#emit(channels, EmitTypes.updated, { after: after.toJSON(), before: before.toJSON() });
    }
    async deleted(channels, data) {
        await this.#emit(channels, EmitTypes.deleted, { before: data.toJSON(), after: null });
    }
    set callers(callers) {
        this.#callers = callers;
        this.#setupSocketConnection();
    }
    register(channel, onJoin) {
        if (!onJoin)
            onJoin = async ({ channel }) => channel;
        this.#routes[channel] = onJoin;
        this.#routes[channel + '/:id'] = onJoin;
        return this;
    }
    #getJoinCb(channel) {
        const matcher = (key) => (0, path_to_regexp_1.match)(key)(channel);
        const matchedChannel = Object.keys(this.#routes).find(matcher) ?? null;
        if (!matchedChannel)
            return null;
        const match = matcher(matchedChannel);
        if (!match)
            return null;
        return {
            onJoin: this.#routes[matchedChannel],
            params: match.params
        };
    }
    async #emit(channels, type, { before, after }) {
        await Promise.all(channels.map(async (channel) => {
            const emitData = { channel, type, before, after };
            await this.#publisher.publish(emitData);
        }));
    }
    #setupSocketConnection = () => {
        const event = 'connection';
        this.#socket.removeAllListeners(event);
        this.#socket.on(event, async (socket) => {
            const socketId = socket.id;
            let user = null;
            if (socket.handshake.auth.token)
                user = await (0, tokens_1.verifyAccessToken)(socket.handshake.auth.token ?? '').catch(() => null);
            socket.on('leave', async (data, callback) => {
                if (!data.channel)
                    return typeof (callback) === 'function' && callback({
                        code: server_1.StatusCodes.ValidationError,
                        message: 'channel is required',
                        channel: ''
                    });
                socket.leave(data.channel);
                return typeof (callback) === 'function' && callback({
                    code: server_1.StatusCodes.Ok,
                    message: '',
                    channel: data.channel
                });
            });
            socket.on('join', async (data, callback) => {
                if (!data.channel)
                    return typeof (callback) === 'function' && callback({
                        code: server_1.StatusCodes.ValidationError,
                        message: 'channel is required',
                        channel: ''
                    });
                const channel = data.channel;
                const route = this.#getJoinCb(channel) ?? null;
                if (!route)
                    return typeof (callback) === 'function' && callback({
                        code: server_1.StatusCodes.BadRequest,
                        message: 'unknown channel',
                        channel
                    });
                const newChannel = await route.onJoin({ channel, user }, route.params, data.query ?? {});
                if (!newChannel)
                    return typeof (callback) === 'function' && callback({
                        code: server_1.StatusCodes.NotAuthorized,
                        message: 'restricted access',
                        channel
                    });
                socket.join(newChannel);
                return typeof (callback) === 'function' && callback({
                    code: server_1.StatusCodes.Ok,
                    message: '',
                    channel: newChannel
                });
            });
            if (user)
                await this.#callers.onConnect(user.id, socketId);
            socket.on('disconnect', async () => {
                if (user)
                    await this.#callers.onDisconnect(user.id, socketId);
            });
        });
    };
}
exports.Listener = Listener;
