"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressServer = void 0;
const utils_1 = require("@fastify/swagger/lib/spec/openapi/utils");
const openapi_1 = __importDefault(require("@wesleytodd/openapi"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
const express_fileupload_1 = __importDefault(require("express-fileupload"));
const express_rate_limit_1 = __importDefault(require("express-rate-limit"));
const helmet_1 = __importDefault(require("helmet"));
const http_1 = __importDefault(require("http"));
const json_schema_resolver_1 = __importDefault(require("json-schema-resolver"));
const pino_http_1 = require("pino-http");
const errors_1 = require("../../errors");
const exit_1 = require("../../exit");
const instance_1 = require("../../instance");
const media_1 = require("../../utils/media");
const middlewares_1 = require("../middlewares");
const requests_1 = require("../requests");
const types_1 = require("../types");
const base_1 = require("./base");
class ExpressServer extends base_1.Server {
    #expressApp;
    #oapi = (0, openapi_1.default)(this.openapiJsonUrl.replace('.json', ''), this.baseOpenapiDoc, { coerce: false });
    #ref = (0, json_schema_resolver_1.default)({ clone: true });
    constructor() {
        const app = (0, express_1.default)();
        super(http_1.default.createServer(app));
        this.#expressApp = app;
        app.disable('x-powered-by');
        if (this.settings.logRequests)
            app.use((0, pino_http_1.pinoHttp)({ logger: instance_1.Instance.get().logger }));
        app.use(express_1.default.json());
        app.use(express_1.default.text());
        app.use((0, cookie_parser_1.default)());
        app.use((0, helmet_1.default)({
            crossOriginResourcePolicy: { policy: 'cross-origin' },
            contentSecurityPolicy: false
        }));
        app.use((0, cors_1.default)({ origin: '*' }));
        app.use(express_1.default.urlencoded({ extended: false }));
        app.use(express_1.default.static(this.staticPath));
        app.use(this.#oapi);
        app.use((0, express_fileupload_1.default)({
            limits: { fileSize: this.settings.maxFileUploadSizeInMb * 1024 * 1024 },
            useTempFiles: false
        }));
        if (this.settings.useRateLimit)
            app.use((0, express_rate_limit_1.default)({
                windowMs: this.settings.rateLimitPeriodInMs,
                limit: this.settings.rateLimit,
                handler: (_, res) => res.status(types_1.StatusCodes.TooManyRequests).json([{ message: 'Too Many Requests' }])
            }));
    }
    registerRoute(route) {
        const openapi = (0, utils_1.prepareOpenapiMethod)(route.schema, this.#ref, this.baseOpenapiDoc, route.path);
        const controllers = [
            ...route.middlewares.map((m) => this.makeMiddleware(m.cb)),
            this.makeController(route.handler)
        ];
        if (!route.schema.hide)
            controllers.unshift(this.#oapi[this.settings.requestSchemaValidation ? 'validPath' : 'path'](openapi), (error, _, __, next) => {
                if ('validationErrors' in error) {
                    const validationErrors = error.validationErrors;
                    throw new errors_1.ValidationError(validationErrors.map((error) => ({
                        messages: [error.message ?? ''],
                        field: error.instancePath.replaceAll('/', '.').split('.').filter(Boolean).join('.')
                    })));
                }
                next();
            });
        if (route.onError)
            controllers.push(this.makeErrorMiddleware(route.onError.cb));
        this.#expressApp[route.method]?.(route.path, ...controllers);
    }
    async startServer(port) {
        this.#expressApp.use(this.makeMiddleware(middlewares_1.notFoundHandler.cb));
        this.#expressApp.use(this.makeErrorMiddleware(middlewares_1.errorHandler.cb));
        return await new Promise((resolve, reject) => {
            try {
                const app = this.server.listen({ host: '0.0.0.0', port }, async () => resolve(true));
                (0, exit_1.addWaitBeforeExit)(app.close);
            }
            catch (err) {
                reject(err);
            }
        });
    }
    async onLoad() { }
    async parse(req) {
        const allHeaders = Object.fromEntries(Object.entries(req.headers).map(([key, val]) => [key, val ?? null]));
        const headers = {
            ...allHeaders,
            AccessToken: req.get('Access-Token'),
            RefreshToken: req.get('Refresh-Token'),
            ContentType: req.get('Content-Type'),
            Referer: req.get('referer'),
            UserAgent: req.get('User-Agent')
        };
        const files = Object.fromEntries(await Promise.all(Object.entries(req.files ?? {}).map(async ([key, file]) => {
            const uploads = Array.isArray(file) ? file : [file];
            const fileArray = await Promise.all(uploads.map(async (f) => ({
                name: f.name,
                type: f.mimetype,
                size: f.size,
                isTruncated: f.truncated,
                data: f.data,
                duration: await (0, media_1.getMediaDuration)(f.data),
            })));
            return [key, fileArray];
        })));
        return req.savedReq ||= new requests_1.Request({
            ip: req.ip,
            body: req.body ?? {},
            cookies: req.cookies ?? {},
            params: req.params ?? {},
            query: req.query ?? {},
            method: req.method,
            path: req.path,
            headers,
            files,
        });
    }
    makeController(cb) {
        return async (req, res, next) => {
            try {
                const request = await this.parse(req);
                const rawResponse = await cb(request);
                const response = rawResponse instanceof requests_1.Response ? rawResponse : request.res({ body: rawResponse });
                if (!response.piped) {
                    Object.entries(response.headers).forEach(([key, value]) => res.header(key, value));
                    const type = response.shouldJSONify ? 'json' : 'send';
                    res.status(response.status)[type](response.body).end();
                }
                else {
                    response.body.pipe(res);
                }
            }
            catch (e) {
                next(e);
            }
        };
    }
    makeMiddleware(cb) {
        return async (req, _res, next) => {
            try {
                await cb(await this.parse(req));
                return next();
            }
            catch (e) {
                return next(e);
            }
        };
    }
    makeErrorMiddleware(cb) {
        return async (err, req, res, _next) => {
            const request = await this.parse(req);
            const rawResponse = await cb(request, err);
            const response = rawResponse instanceof requests_1.Response ? rawResponse : request.res({ body: rawResponse, status: types_1.StatusCodes.BadRequest });
            if (!response.piped) {
                Object.entries(response.headers).forEach(([key, value]) => value && res.header(key, value));
                res.status(response.status).send(response.body).end();
            }
            else {
                response.body.pipe(res);
            }
        };
    }
}
exports.ExpressServer = ExpressServer;
