"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FastifyServer = void 0;
const cookie_1 = __importDefault(require("@fastify/cookie"));
const cors_1 = __importDefault(require("@fastify/cors"));
const formbody_1 = __importDefault(require("@fastify/formbody"));
const helmet_1 = __importDefault(require("@fastify/helmet"));
const multipart_1 = __importDefault(require("@fastify/multipart"));
const rate_limit_1 = __importDefault(require("@fastify/rate-limit"));
const static_1 = __importDefault(require("@fastify/static"));
const swagger_1 = __importDefault(require("@fastify/swagger"));
const fastify_1 = __importDefault(require("fastify"));
const qs_1 = __importDefault(require("qs"));
const errors_1 = require("../../errors");
const exit_1 = require("../../exit");
const instance_1 = require("../../instance");
const media_1 = require("../../utils/media");
const middlewares_1 = require("../middlewares");
const requests_1 = require("../requests");
const types_1 = require("../types");
const base_1 = require("./base");
function getFastifyApp() {
    const instance = instance_1.Instance.get();
    return (0, fastify_1.default)({
        ignoreTrailingSlash: true,
        caseSensitive: false,
        disableRequestLogging: !instance.settings.logRequests,
        loggerInstance: instance.settings.logRequests ? instance.logger : undefined,
        ajv: { customOptions: { coerceTypes: false } },
        schemaErrorFormatter: (errors, data) => new errors_1.ValidationError(errors.map((error) => ({ messages: [error.message ?? ''], field: `${data}${error.instancePath}`.replaceAll('/', '.') })))
    });
}
class FastifyServer extends base_1.Server {
    #fastifyApp;
    constructor() {
        const app = getFastifyApp();
        super(app.server);
        this.#fastifyApp = app;
        app.decorateRequest('savedReq', null);
        app.register(static_1.default, { root: this.staticPath });
        app.register(cookie_1.default, {});
        app.register(cors_1.default, { origin: '*' });
        app.register(swagger_1.default, { openapi: this.baseOpenapiDoc });
        app.get(this.openapiJsonUrl, (_, res) => {
            res.code(200).send(app.swagger({}));
        });
        app.register(formbody_1.default, { parser: (str) => qs_1.default.parse(str) });
        app.register(helmet_1.default, { crossOriginResourcePolicy: { policy: 'cross-origin' }, contentSecurityPolicy: false });
        app.register(multipart_1.default, {
            attachFieldsToBody: 'keyValues',
            throwFileSizeLimit: false,
            limits: { fileSize: this.settings.maxFileUploadSizeInMb * 1024 * 1024 },
            onFile: async (f) => {
                const buffer = await f.toBuffer();
                const parsed = {
                    name: f.filename,
                    type: f.mimetype,
                    size: buffer.byteLength,
                    isTruncated: f.file.truncated,
                    data: buffer,
                    duration: await (0, media_1.getMediaDuration)(buffer),
                };
                f.value = parsed;
            }
        });
        if (this.settings.useRateLimit)
            app.register(rate_limit_1.default, {
                max: this.settings.rateLimit,
                timeWindow: this.settings.rateLimitPeriodInMs,
                errorResponseBuilder: (_, context) => ({
                    statusCode: types_1.StatusCodes.TooManyRequests,
                    message: JSON.stringify([{ message: `Too Many Requests. Retry in ${context.after}` }])
                })
            });
        if (!this.settings.requestSchemaValidation) {
            app.setValidatorCompiler(() => () => true);
            app.setSerializerCompiler(() => (data) => JSON.stringify(data));
        }
    }
    async onLoad() {
        await this.#fastifyApp.ready();
    }
    registerRoute(route) {
        this.#fastifyApp.register(async (inst) => {
            inst.route({
                url: route.path,
                method: route.method,
                handler: this.makeController(route.handler),
                preHandler: route.middlewares.map((m) => this.makeMiddleware(m.cb)),
                errorHandler: route.onError ? this.makeErrorMiddleware(route.onError.cb) : undefined,
                schema: route.schema,
            });
        });
    }
    async startServer(port) {
        this.#fastifyApp.setNotFoundHandler(this.makeController(middlewares_1.notFoundHandler.cb));
        this.#fastifyApp.setErrorHandler(this.makeErrorMiddleware(middlewares_1.errorHandler.cb));
        await this.#fastifyApp.listen({ port, host: '0.0.0.0' });
        (0, exit_1.addWaitBeforeExit)(this.#fastifyApp.close);
        return true;
    }
    async parse(req) {
        const allHeaders = Object.fromEntries(Object.entries(req.headers).map(([key, val]) => [key, val ?? null]));
        const headers = {
            ...allHeaders,
            AccessToken: req.headers['access-token']?.toString(),
            RefreshToken: req.headers['refresh-token']?.toString(),
            ContentType: req.headers['content-type']?.toString(),
            Referer: req.headers['referer']?.toString(),
            UserAgent: req.headers['user-agent']?.toString()
        };
        const { body, files } = excludeBufferKeys(req.body ?? {});
        return req.savedReq ||= new requests_1.Request({
            ip: req.ip,
            body,
            cookies: req.cookies ?? {},
            params: req.params ?? {},
            query: req.query ?? {},
            method: req.method,
            path: req.url,
            headers,
            files,
        });
    }
    makeController(cb) {
        const handler = async (req, reply) => {
            const request = await this.parse(req);
            const rawResponse = await cb(request);
            const response = rawResponse instanceof requests_1.Response ? rawResponse : request.res({ body: rawResponse });
            return reply.status(response.status).headers(response.headers).send(response.body);
        };
        return handler;
    }
    makeMiddleware(cb) {
        const handler = async (req) => {
            await cb(await this.parse(req));
        };
        return handler;
    }
    makeErrorMiddleware(cb) {
        const handler = async (error, req, reply) => {
            const request = await this.parse(req);
            const rawResponse = await cb(request, error);
            const response = rawResponse instanceof requests_1.Response ? rawResponse : request.res({ body: rawResponse, status: types_1.StatusCodes.BadRequest });
            return reply.status(response.status).headers(response.headers).send(response.body);
        };
        return handler;
    }
}
exports.FastifyServer = FastifyServer;
function excludeBufferKeys(body) {
    if (typeof body !== 'object')
        return { body, files: {} };
    const entries = Object.entries(body ?? {});
    const isFile = (val) => Array.isArray(val) ? isFile(val.at(0)) : Buffer.isBuffer(val?.data);
    const fileEntries = entries.filter(([_, value]) => isFile(value)).map(([key, value]) => [key, Array.isArray(value) ? value : [value]]);
    const nonFileEntries = entries.filter(([_, value]) => !isFile(value));
    return {
        body: Object.fromEntries(nonFileEntries),
        files: Object.fromEntries(fileEntries)
    };
}
