import { Enum } from '../enums/types';
import { CustomError } from '../errors';
import { FastifySchema } from 'fastify';
import { Defined, Flatten, IsInTypeList, IsType, JSONValue } from '../types';
import type { Request, Response } from './requests';
export declare const Methods: {
    readonly get: "get";
    readonly post: "post";
    readonly put: "put";
    readonly patch: "patch";
    readonly delete: "delete";
};
export type MethodTypes = Enum<typeof Methods>;
export declare const StatusCodes: {
    readonly Ok: 200;
    readonly Found: 302;
    readonly BadRequest: 400;
    readonly NotAuthenticated: 401;
    readonly NotAuthorized: 403;
    readonly NotFound: 404;
    readonly ValidationError: 422;
    readonly TooManyRequests: 429;
    readonly AccessTokenExpired: 461;
};
export type SupportedStatusCodes = Enum<typeof StatusCodes>;
export type GoodStatusCodes = 200 | 302;
export type BadStatusCodes = Exclude<SupportedStatusCodes, GoodStatusCodes>;
type ApiErrors = Record<BadStatusCodes, JSONValue<CustomError['serializedErrors']>>;
type ApiResponse<T, StatusCode extends SupportedStatusCodes> = Record<StatusCode, JSONValue<T>> | Omit<ApiErrors, StatusCode>;
export interface Api<Res = any, Key extends string = string, Method extends MethodTypes = MethodTypes, Body = any, Params extends Record<string, string> = Record<string, string>, Query extends Record<string, any> = Record<string, any>, RequestHeaders extends HeadersType = HeadersType, ResponseHeaders extends HeadersType = HeadersType, DefaultStatus extends SupportedStatusCodes = SupportedStatusCodes> {
    key: Key;
    method: Method;
    response: Res;
    body?: Body;
    params?: Params;
    query?: Query;
    requestHeaders?: RequestHeaders;
    responseHeaders?: ResponseHeaders;
    defaultStatusCode?: DefaultStatus;
}
export type HeadersType = Record<string, string | string[] | undefined>;
export type FileSchema = 'equipped-file-schema';
export interface ApiDef<T extends Api> {
    key: T['key'];
    method: T['method'];
    body: Flatten<GetApiPart<T, 'body', false>>;
    params: Flatten<GetApiPart<T, 'params', false>>;
    query: Flatten<GetApiPart<T, 'query', false>>;
    requestHeaders: Flatten<GetApiPart<T, 'requestHeaders', false>>;
    responseHeaders: Flatten<GetApiPart<T, 'responseHeaders', false>>;
    responses: Flatten<ApiResponse<T['response'], GetApiPart<T, 'defaultStatusCode', true, 200>>>;
    __apiDef: true;
}
type Awaitable<T> = Promise<T> | T;
type Res<T, S extends SupportedStatusCodes, H extends HeadersType> = Awaitable<IsInTypeList<S, [SupportedStatusCodes, 200, unknown]> extends true ? IsInTypeList<H, [HeadersType, unknown]> extends true ? Response<T, S, H> | T : Response<T, S, H> : Response<T, S, H>>;
type InferApiFromApiDef<T> = T extends ApiDef<infer A> ? A : never;
type ExcludeUnknown<T, D> = IsType<T, unknown> extends true ? D : T;
export type GetApiPart<T extends Api, K extends keyof Api, Def extends boolean = true, Default extends Api[K] = Def extends true ? Defined<Api[K]> : Api[K]> = IsInTypeList<K, ['key', 'method', 'body']> extends true ? T[K] : Def extends true ? ExcludeUnknown<Defined<T[K]>, Default> : ExcludeUnknown<T[K], Default>;
export type RouteHandler<Def extends Api = Api> = (req: Request<Def>) => Res<Def['response'], GetApiPart<Def, 'defaultStatusCode'>, GetApiPart<Def, 'responseHeaders'>>;
export type ErrorHandler<Def extends Api = Api> = (req: Request<Def>, err: Error) => Res<CustomError['serializedErrors'], CustomError['statusCode'], HeadersType>;
export type RouteMiddlewareHandler<Def extends Api = Api> = (req: Request<Def>) => Awaitable<void>;
export type HandlerSetup = (route: Route) => void;
export type RouteSchema = Omit<FastifySchema, 'tags' | 'security' | 'hide' | 'description'> & {
    descriptions?: string[];
    title?: string;
};
type RouteGroup = {
    name: string;
    description?: string;
};
export interface Route<Def extends ApiDef<Api> = ApiDef<Api>> {
    key?: Def['key'];
    path: string;
    method: Def['method'];
    handler: RouteHandler<InferApiFromApiDef<Def>>;
    onSetupHandler?: HandlerSetup;
    middlewares?: ReturnType<typeof makeMiddleware>[];
    onError?: ReturnType<typeof makeErrorMiddleware>;
    schema?: RouteSchema;
    groups?: (RouteGroup | RouteGroup['name'])[];
    descriptions?: string[];
    hideSchema?: boolean;
    security?: Record<string, string[]>[];
}
export type RouteConfig<T extends ApiDef<Api> = ApiDef<Api>> = Omit<Route<T>, 'method' | 'handler'>;
export type GeneralConfig = Omit<RouteConfig, 'schema' | 'key'>;
export type AddMethodImpls = {
    [Method in MethodTypes]: <T extends ApiDef<any>>(route: RouteConfig<T>) => (handler: RouteHandler<InferApiFromApiDef<T>>) => Route<T>;
};
declare class MiddlewareHandler<Cb extends Function> {
    cb: Cb;
    onSetup?: HandlerSetup | undefined;
    private constructor();
    static make<Cb extends Function>(cb: Cb, onSetup?: HandlerSetup): MiddlewareHandler<Cb>;
}
export declare const makeMiddleware: <Def extends Api = Api<any, string, MethodTypes, any, Record<string, string>, Record<string, any>, HeadersType, HeadersType, SupportedStatusCodes>>(cb: RouteMiddlewareHandler<Def>, onSetup?: HandlerSetup | undefined) => MiddlewareHandler<RouteMiddlewareHandler<Def>>;
export declare const makeErrorMiddleware: <Def extends Api = Api<any, string, MethodTypes, any, Record<string, string>, Record<string, any>, HeadersType, HeadersType, SupportedStatusCodes>>(cb: ErrorHandler<Def>, onSetup?: HandlerSetup | undefined) => MiddlewareHandler<ErrorHandler<Def>>;
export {};
